from django.conf import settings
from django.db import models
from uuid import uuid4


TXN_TYPE_CHOICES = [(s, s) for s in ['Purchase', 'Auth', 'Complete']]
CURRENCY_CHOICES = [('AUD', "Australian Dollar"),
                    ('BND', "Brunei Dollar"),
                    ('CAD', "Canadian Dollar"),
                    ('CHF', "Switzerland Franc"),
                    ('EUR', "Euro"),
                    ('FJD', "Fiji Dollar"),
                    ('FRF', "French Franc"),
                    ('GBP', "United Kingdom Pound"),
                    ('HKD', "Hong Kong Dollar"),
                    ('INR', "Indian Rupee"),
                    ('JPY', "Japanese Yen"),
                    ('KWD', "Kuwait Dinar"),
                    ('MYR', "Malaysian Ringgit"),
                    ('NZD', "New Zealand  Dollar"),
                    ('PGK', "Papua New Guinean Kina"),
                    ('SBD', "Solomon Islander Dollar"),
                    ('SGD', "Singapore Dollar"),
                    ('THB', "Thai Baht"),
                    ('TOP', "Tongan Pa'anga"),
                    ('USD', "United States Dollar"),
                    ('VUV', "Vanuatu Vatu"),
                    ('WST', "Samoan Tala"),
                    ('ZAR', "South African Rand")]
STATE_CHOICES = [(s, s) for s in ['Init', 'GenerateRequest', 'RequestSent',
                                  'ProcessResponse', 'Complete']]


class Transaction(models.Model):

    # Model field names match API fields.
    TxnId = models.CharField(max_length=16, editable=False, primary_key=True)
    TxnType = models.CharField(max_length=8, choices=TXN_TYPE_CHOICES)
    MerchantReference = models.CharField(max_length=64, null=True, blank=True)
    TxnData1 = models.CharField(max_length=32, null=True, blank=True)
    TxnData2 = models.CharField(max_length=32, null=True, blank=True)
    TxnData3 = models.CharField(max_length=32, null=True, blank=True)
    AmountInput = models.DecimalField(decimal_places=2, max_digits=12)
    AmountSettlement = models.DecimalField(decimal_places=2, max_digits=12,
                                           null=True, blank=True)
    CurrencyInput = models.CharField(max_length=4, choices=CURRENCY_CHOICES)
    CurrencySettlement = models.CharField(max_length=4,
                                          choices=CURRENCY_CHOICES,
                                          null=True, blank=True)
    EnableAddBillCard = models.IntegerField(null=True, blank=True)
    BillingId = models.CharField(max_length=32, null=True, blank=True)
    DpsBillingId = models.CharField(max_length=16, null=True, blank=True)
    Opt = models.CharField(max_length=64, null=True, blank=True)
    AuthCode = models.CharField(max_length=22, null=True, blank=True)
    DpsTxnRef = models.CharField(max_length=16, null=True, blank=True)
    Success = models.IntegerField(null=True, blank=True)
    ResponseText = models.TextField(max_length=32, null=True, blank=True)
    Response = models.TextField(null=True, blank=True)
    ClientInfo = models.CharField(max_length=15, null=True, blank=True)
    # No info on max_length given in the documentation.
    TxnMac = models.TextField(null=True, blank=True)
    # Service fields.
    created = models.DateTimeField(auto_now_add=True)
    state = models.CharField(max_length=15, choices=STATE_CHOICES,
                             default='Init')
    complete = models.BooleanField(default=False)

    def __init__(self, *args, **kwargs):
        # If no txn_id provided generate a new one.
        if not kwargs.get('TxnId'):
            kwargs['TxnId'] = self._generate_unique_txnid()
        # Currency code must be provided
        # either within transaction or in settings.py.
        if not kwargs.get('CurrencyInput'):
            try:
                kwargs['CurrencyInput'] = getattr(settings, 'PXPAY_CURRENCY')
            except AttributeError:
                raise KeyError('No currency set. Please provide CurrencyInput \
                               or specify PXPAY_CURRENCY in settings')
        return super(Transaction, self).__init__(*args, **kwargs)

    def __unicode__(self):
        return '%s' % self.TxnId

    def _generate_unique_txnid(self):
        TxnId = str(uuid4())[:15].replace('-', '')
        while self._base_manager.filter(TxnId=TxnId).exists():
            TxnId = str(uuid4())[:15].replace('-', '')
        return TxnId

    class Meta:

        ordering = ('-created', )



from django.conf import settings
from pxpay.models import CURRENCY_CHOICES, TXN_TYPE_CHOICES, Transaction
from xml.dom.minidom import parseString, Document
import re
import requests


class Request(object):
    """
    Represents a GenerateRequest request.
    First request to PaymentExpress of two.
    """

    ROOT_ELEMENT = 'GenerateRequest'
    REQUEST_FILEDS = [
        'TxnId',
        'TxnType',
        'MerchantReference',
        'TxnData1',
        'TxnData2',
        'TxnData3',
        'AmountInput',
        'CurrencyInput',
        'EnableAddBillCard',
        'BillingId',
        'Opt',
        'UrlFail',
        'UrlSuccess']

    def __init__(self, userid, passkey, txn, kwargs):
        txn.state = 'GenerateRequest'
        txn.save()
        self.data = {}
        self.set_auth(userid, passkey)
        for element in txn._meta.get_all_field_names():
            if element in self.REQUEST_FILEDS:
                self.set_element(element, getattr(txn, element, None))
        for element in kwargs:
            if element in self.REQUEST_FILEDS:
                self.set_element(element, kwargs[element])

    @property
    def request_xml(self):
        doc = Document()
        root = doc.createElement(self.ROOT_ELEMENT)
        doc.appendChild(root)
        for key, value in self.data.items():
            self._create_element(doc, root, key, value=value)
        return doc.toxml()

    def set_auth(self, userid, passkey):
        self.set_element('PxPayUserId', userid)
        self.set_element('PxPayKey', passkey)

    def set_element(self, name, value):
        self.data[name] = value

    def _create_element(self, doc, parent, tag, value=None, attributes=None):
        ele = doc.createElement(tag)
        parent.appendChild(ele)
        if value:
            text = doc.createTextNode(str(value))
            ele.appendChild(text)
        if attributes:
            [ele.setAttribute(k, v) for k, v in attributes.items()]
        return ele

    def __unicode__(self):
        return self.request_xml


class ProcessResponse(Request):
    """
    A ProcessResponse Request. The second subsequent request.
    """

    ROOT_ELEMENT = 'ProcessResponse'

    def __init__(self, userid, passkey, kwargs):
        self.data = {}
        self.set_auth(userid, passkey)
        self.set_element('Response', kwargs.get('Response'))


class Response(object):
    """
    Encapsulate a PxPay response.
    """

    RESPONSE_FILEDS = [
        'AmountSettlement',
        'AuthCode',
        'DpsTxnRef',
        'Success',
        'ResponseText',
        'DpsBillingId',
        'CurrencySettlement',
        'ClientInfo',
        'TxnMac',
        'BillingId']

    def __init__(self, request_xml, response_xml, txn=None):
        self.request_xml = request_xml
        self.response_xml = response_xml
        self.response_parsed = self._extract_data(self.response_xml)
        if not txn:
            txn = Transaction.objects.get(TxnId=self.get_data['TxnId'])
        for element in self.response_parsed.firstChild.childNodes:
            if element.nodeName in self.RESPONSE_FILEDS:
                val = self._get_element_val(element)
                if val is not None and val != '':
                    setattr(txn, element.nodeName, val)
        txn.save()

    def _extract_data(self, response_xml):
        if (response_xml == '' or
            response_xml == '<?xml version="1.0" ?>' or
            response_xml is None):
            return None
        return parseString(response_xml)

    @property
    def get_data(self):
        if self.is_valid:
            data = {}
            for element in self.response_parsed.firstChild.childNodes:
                data[element.nodeName] = self._get_element_val(element)
            return data
        return None

    def _get_element_val(self, element):
        if element.firstChild:
            return element.firstChild.data
        return None

    @property
    def is_valid(self):
        ele = self.response_parsed.firstChild
        if ele is None or ele.attributes is None:
            return False
        return int(ele.attributes.get('valid').value) == 1


class Gateway(object):
    """
    Transport class and the entry point.
    """

    def __init__(self, **kwargs):
        self.pxpay_url = 'https://sec2.paymentexpress.com/pxpay/pxaccess.aspx'
        try:
            self.userid = kwargs.get('PXPAY_USERID',
                                     getattr(settings, 'PXPAY_USERID'))
        except AttributeError:
            raise KeyError('No PXPAY_USERID set. Please provide PXPAY_USERID \
                           as an argument or specify it in settings')
        try:
            self.passkey = kwargs.get('PXPAY_KEY',
                                      getattr(settings, 'PXPAY_KEY'))
        except AttributeError:
            raise KeyError('No PXPAY_KEY set. Please provide PXPAY_KEY as an \
                           argument or specify it in settings')

    def _fetch_response(self, request, txn=None):
        response = requests.post(self.pxpay_url, request.request_xml,
                                 verify=False)
        return Response(request.request_xml, response.text, txn=txn)

    def transaction(self, **kwargs):
        if kwargs.get('TxnId'):
            txn = Transaction.objects.get(TxnId__exact=kwargs.get('TxnId'))
        else:
            return Transaction.objects.create(**kwargs)
        # Generate a new transaction if attributes had changed
        # before the transaction was completed.
        if not txn.complete:
            for key, value in kwargs.iteritems():
                if key != 'TxnId' and getattr(txn, key) != value:
                    del kwargs['TxnId']
                    txn = Transaction.objects.create(**kwargs)
                    break
        txn.save()
        return txn

    def process_transaction(self, txn, **kwargs):
        """
        Sends payment request
        """
        request = Request(self.userid, self.passkey, txn, kwargs)
        ret = self._fetch_response(request, txn=txn)
        txn.state = 'RequestSent'
        txn.save()
        return ret

    def process_response(self, **kwargs):
        """
        Post-processing transaction validation.
        """
        request = ProcessResponse(self.userid, self.passkey, kwargs)
        ret = self._fetch_response(request)
        txn = Transaction.objects.get(TxnId=ret.get_data['TxnId'])
        txn.state = 'Complete'
        txn.complete = True
        txn.save()
        return ret



#!/usr/bin/env python
import os
from setuptools import setup, find_packages


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name='django-pxpay',
    version='0.2.2',
    url='http://github.com/pavel-shpilev/django-pxpay',
    author="Pavel Shpilev",
    author_email="p.shpilev@gmail.com",
    description="PaymentExpress PxPay Gateway for Django",
    long_description=read('README.rst'),
    keywords="PxPay, Payment, PaymentExpress, DPS, Django, Python",
    license='BSD',
    packages=find_packages(exclude=['test']),
    install_requires=['requests>=0.13.5', 'django>=1.4'],
    include_package_data=True,
    classifiers=['Environment :: Web Environment',
                 'Framework :: Django',
                 'Intended Audience :: Developers',
                 'License :: OSI Approved :: BSD License',
                 'Operating System :: OS Independent',
                 'Programming Language :: Python'])
